# CakePHP API Pagination

[![Latest Version on Packagist][ico-version]][link-packagist]
[![Software License][ico-license]](LICENSE.md)
[![Build Status][ico-travis]][link-travis]
[![Coverage Status][ico-scrutinizer]][link-scrutinizer]
[![Quality Score][ico-code-quality]][link-code-quality]
[![Total Downloads][ico-downloads]][link-downloads]

This is a simple component for CakePHP 3 which injects pagination information
from CakePHP's Paginator, into Json and Xml View responses.

## Install

Via Composer

``` bash
$ composer require bcrowe/cakephp-api-pagination
```

Then make sure to load the plugin in your application's `bootstrap.php` file:

``` php
Plugin::load('BryanCrowe/ApiPagination');
```

## Usage

Load the component in a controller's `initialize()` method:


``` php
public function initialize()
{
    parent::initialize();
    $this->loadComponent('BryanCrowe/ApiPagination.ApiPagination');
}
```

Your responses will now contain the pagination information, and will look
something like:

``` json
{
    "articles": ["...", "...", "..."],
    "pagination": {
        "finder": "all",
        "page": 1,
        "current": 20,
        "count": 5000,
        "perPage": 20,
        "prevPage": false,
        "nextPage": true,
        "pageCount": 250,
        "sort": null,
        "direction": false,
        "limit": null,
        "sortDefault": false,
        "directionDefault": false
    }
}
```

### Configuring the Pagination Ouput

ApiPagination has three keys for configuration: `key`, `aliases`, and `visible`.

The `key` configuration key allows you to change the name of the pagination key.
For instance, if you wanted to change "pagination" to "paging".

The `aliases` configuration key allows you to change names of the pagination
keys. For instance, if you wanted to change "page" to "currentPage".

The `visible` confiration key allows you to set which pagination keys will be
visible in your rendered response.

An example using all these configuration keys:

``` php
$this->loadComponent('BryanCrowe/ApiPagination.ApiPagination', [
    'key' => 'paging',
    'aliases' => [
        'page' => 'currentPage',
        'current' => 'resultCount'
    ],
    'visible' => [
        'currentPage',
        'resultCount',
        'prevPage',
        'nextPage'
    ]
]);
```

This configuration would yield:

``` json
{
    "articles": ["...", "...", "..."],
    "paging": {
        "prevPage": false,
        "nextPage": true,
        "currentPage": 1,
        "resultCount": 20
    }
}
```

**Note:** Whenever setting key visibily, use the aliased name if you've given it
one.

## Change log

Please see [CHANGELOG](CHANGELOG.md) for more information what has changed recently.

## Testing

``` bash
$ composer test
```

## Contributing

Please see [CONTRIBUTING](CONTRIBUTING.md) for details.

## Security

If you discover any security related issues, please email bryan@bryan-crowe.com instead of using the issue tracker.

## Credits

- [Bryan Crowe][link-author]
- [All Contributors][link-contributors]

## License

The MIT License (MIT). Please see [License File](LICENSE.md) for more information.

[ico-version]: https://img.shields.io/packagist/v/bcrowe/cakephp-api-pagination.svg?style=flat-square
[ico-license]: https://img.shields.io/badge/license-MIT-brightgreen.svg?style=flat-square
[ico-travis]: https://img.shields.io/travis/bcrowe/cakephp-api-pagination/master.svg?style=flat-square
[ico-scrutinizer]: https://img.shields.io/scrutinizer/coverage/g/bcrowe/cakephp-api-pagination.svg?style=flat-square
[ico-code-quality]: https://img.shields.io/scrutinizer/g/bcrowe/cakephp-api-pagination.svg?style=flat-square
[ico-downloads]: https://img.shields.io/packagist/dt/bcrowe/cakephp-api-pagination.svg?style=flat-square

[link-packagist]: https://packagist.org/packages/bcrowe/cakephp-api-pagination
[link-travis]: https://travis-ci.org/bcrowe/cakephp-api-pagination
[link-scrutinizer]: https://scrutinizer-ci.com/g/bcrowe/cakephp-api-pagination/code-structure
[link-code-quality]: https://scrutinizer-ci.com/g/bcrowe/cakephp-api-pagination
[link-downloads]: https://packagist.org/packages/bcrowe/cakephp-api-pagination
[link-author]: https://github.com/bcrowe
[link-contributors]: ../../contributors
